
import { localize2 } from '@codingame/monaco-vscode-api/vscode/vs/nls';
import { GroupLocation, GroupDirection } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/editor/common/editorGroupsService';
import { IEditorGroupsService } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/editor/common/editorGroupsService.service';
import { Parts } from '@codingame/monaco-vscode-c8227507-8e59-53d6-b50b-71c0ab384cf4-common/vscode/vs/workbench/services/layout/browser/layoutService';
import { IWorkbenchLayoutService } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/layout/browser/layoutService.service';
import { Action2, registerAction2 } from '@codingame/monaco-vscode-api/vscode/vs/platform/actions/common/actions';
import { Categories } from '@codingame/monaco-vscode-api/vscode/vs/platform/action/common/actionCommonCategories';
import { Direction } from '@codingame/monaco-vscode-6980eeab-47bb-5a48-8e15-32caf0785565-common/vscode/vs/base/browser/ui/grid/grid';
import { KeyCode, KeyMod } from '@codingame/monaco-vscode-api/vscode/vs/base/common/keyCodes';
import { IEditorService } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/editor/common/editorService.service';
import { IPaneCompositePartService } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/panecomposite/browser/panecomposite.service';
import { ViewContainerLocation } from '@codingame/monaco-vscode-api/vscode/vs/workbench/common/views';
import { KeybindingWeight } from '@codingame/monaco-vscode-api/vscode/vs/platform/keybinding/common/keybindingsRegistry';
import { getActiveWindow } from '@codingame/monaco-vscode-api/vscode/vs/base/browser/dom';
import { isAuxiliaryWindow } from '@codingame/monaco-vscode-api/vscode/vs/base/browser/window';

class BaseNavigationAction extends Action2 {
    constructor(options, direction) {
        super(options);
        this.direction = direction;
    }
    run(accessor) {
        const layoutService = accessor.get(IWorkbenchLayoutService);
        const editorGroupService = accessor.get(IEditorGroupsService);
        const paneCompositeService = accessor.get(IPaneCompositePartService);
        const isEditorFocus = layoutService.hasFocus(Parts.EDITOR_PART);
        const isPanelFocus = layoutService.hasFocus(Parts.PANEL_PART);
        const isSidebarFocus = layoutService.hasFocus(Parts.SIDEBAR_PART);
        const isAuxiliaryBarFocus = layoutService.hasFocus(Parts.AUXILIARYBAR_PART);
        let neighborPart;
        if (isEditorFocus) {
            const didNavigate = this.navigateAcrossEditorGroup(this.toGroupDirection(this.direction), editorGroupService);
            if (didNavigate) {
                return;
            }
            neighborPart = layoutService.getVisibleNeighborPart(Parts.EDITOR_PART, this.direction);
        }
        if (isPanelFocus) {
            neighborPart = layoutService.getVisibleNeighborPart(Parts.PANEL_PART, this.direction);
        }
        if (isSidebarFocus) {
            neighborPart = layoutService.getVisibleNeighborPart(Parts.SIDEBAR_PART, this.direction);
        }
        if (isAuxiliaryBarFocus) {
            neighborPart = neighborPart = layoutService.getVisibleNeighborPart(Parts.AUXILIARYBAR_PART, this.direction);
        }
        if (neighborPart === Parts.EDITOR_PART) {
            if (!this.navigateBackToEditorGroup(this.toGroupDirection(this.direction), editorGroupService)) {
                this.navigateToEditorGroup(this.direction === Direction.Right ? GroupLocation.FIRST : GroupLocation.LAST, editorGroupService);
            }
        }
        else if (neighborPart === Parts.SIDEBAR_PART) {
            this.navigateToSidebar(layoutService, paneCompositeService);
        }
        else if (neighborPart === Parts.PANEL_PART) {
            this.navigateToPanel(layoutService, paneCompositeService);
        }
        else if (neighborPart === Parts.AUXILIARYBAR_PART) {
            this.navigateToAuxiliaryBar(layoutService, paneCompositeService);
        }
    }
    async navigateToPanel(layoutService, paneCompositeService) {
        if (!layoutService.isVisible(Parts.PANEL_PART)) {
            return false;
        }
        const activePanel = paneCompositeService.getActivePaneComposite(ViewContainerLocation.Panel);
        if (!activePanel) {
            return false;
        }
        const activePanelId = activePanel.getId();
        const res = await paneCompositeService.openPaneComposite(activePanelId, ViewContainerLocation.Panel, true);
        if (!res) {
            return false;
        }
        return res;
    }
    async navigateToSidebar(layoutService, paneCompositeService) {
        if (!layoutService.isVisible(Parts.SIDEBAR_PART)) {
            return false;
        }
        const activeViewlet = paneCompositeService.getActivePaneComposite(ViewContainerLocation.Sidebar);
        if (!activeViewlet) {
            return false;
        }
        const activeViewletId = activeViewlet.getId();
        const viewlet = await paneCompositeService.openPaneComposite(activeViewletId, ViewContainerLocation.Sidebar, true);
        return !!viewlet;
    }
    async navigateToAuxiliaryBar(layoutService, paneCompositeService) {
        if (!layoutService.isVisible(Parts.AUXILIARYBAR_PART)) {
            return false;
        }
        const activePanel = paneCompositeService.getActivePaneComposite(ViewContainerLocation.AuxiliaryBar);
        if (!activePanel) {
            return false;
        }
        const activePanelId = activePanel.getId();
        const res = await paneCompositeService.openPaneComposite(activePanelId, ViewContainerLocation.AuxiliaryBar, true);
        if (!res) {
            return false;
        }
        return res;
    }
    navigateAcrossEditorGroup(direction, editorGroupService) {
        return this.doNavigateToEditorGroup({ direction }, editorGroupService);
    }
    navigateToEditorGroup(location, editorGroupService) {
        return this.doNavigateToEditorGroup({ location }, editorGroupService);
    }
    navigateBackToEditorGroup(direction, editorGroupService) {
        if (!editorGroupService.activeGroup) {
            return false;
        }
        const oppositeDirection = this.toOppositeDirection(direction);
        const groupInBetween = editorGroupService.findGroup({ direction: oppositeDirection }, editorGroupService.activeGroup);
        if (!groupInBetween) {
            editorGroupService.activeGroup.focus();
            return true;
        }
        return false;
    }
    toGroupDirection(direction) {
        switch (direction) {
            case Direction.Down: return GroupDirection.DOWN;
            case Direction.Left: return GroupDirection.LEFT;
            case Direction.Right: return GroupDirection.RIGHT;
            case Direction.Up: return GroupDirection.UP;
        }
    }
    toOppositeDirection(direction) {
        switch (direction) {
            case GroupDirection.UP: return GroupDirection.DOWN;
            case GroupDirection.RIGHT: return GroupDirection.LEFT;
            case GroupDirection.LEFT: return GroupDirection.RIGHT;
            case GroupDirection.DOWN: return GroupDirection.UP;
        }
    }
    doNavigateToEditorGroup(scope, editorGroupService) {
        const targetGroup = editorGroupService.findGroup(scope, editorGroupService.activeGroup);
        if (targetGroup) {
            targetGroup.focus();
            return true;
        }
        return false;
    }
}
registerAction2(class extends BaseNavigationAction {
    constructor() {
        super({
            id: 'workbench.action.navigateLeft',
            title: ( localize2(2750, 'Navigate to the View on the Left')),
            category: Categories.View,
            f1: true
        }, Direction.Left);
    }
});
registerAction2(class extends BaseNavigationAction {
    constructor() {
        super({
            id: 'workbench.action.navigateRight',
            title: ( localize2(2751, 'Navigate to the View on the Right')),
            category: Categories.View,
            f1: true
        }, Direction.Right);
    }
});
registerAction2(class extends BaseNavigationAction {
    constructor() {
        super({
            id: 'workbench.action.navigateUp',
            title: ( localize2(2752, 'Navigate to the View Above')),
            category: Categories.View,
            f1: true
        }, Direction.Up);
    }
});
registerAction2(class extends BaseNavigationAction {
    constructor() {
        super({
            id: 'workbench.action.navigateDown',
            title: ( localize2(2753, 'Navigate to the View Below')),
            category: Categories.View,
            f1: true
        }, Direction.Down);
    }
});
class BaseFocusAction extends Action2 {
    constructor(options, focusNext) {
        super(options);
        this.focusNext = focusNext;
    }
    run(accessor) {
        const layoutService = accessor.get(IWorkbenchLayoutService);
        const editorService = accessor.get(IEditorService);
        this.focusNextOrPreviousPart(layoutService, editorService, this.focusNext);
    }
    findVisibleNeighbour(layoutService, part, next) {
        const activeWindow = getActiveWindow();
        const windowIsAuxiliary = isAuxiliaryWindow(activeWindow);
        let neighbour;
        if (windowIsAuxiliary) {
            switch (part) {
                case Parts.EDITOR_PART:
                    neighbour = Parts.STATUSBAR_PART;
                    break;
                default:
                    neighbour = Parts.EDITOR_PART;
            }
        }
        else {
            switch (part) {
                case Parts.EDITOR_PART:
                    neighbour = next ? Parts.PANEL_PART : Parts.SIDEBAR_PART;
                    break;
                case Parts.PANEL_PART:
                    neighbour = next ? Parts.AUXILIARYBAR_PART : Parts.EDITOR_PART;
                    break;
                case Parts.AUXILIARYBAR_PART:
                    neighbour = next ? Parts.STATUSBAR_PART : Parts.PANEL_PART;
                    break;
                case Parts.STATUSBAR_PART:
                    neighbour = next ? Parts.ACTIVITYBAR_PART : Parts.AUXILIARYBAR_PART;
                    break;
                case Parts.ACTIVITYBAR_PART:
                    neighbour = next ? Parts.SIDEBAR_PART : Parts.STATUSBAR_PART;
                    break;
                case Parts.SIDEBAR_PART:
                    neighbour = next ? Parts.EDITOR_PART : Parts.ACTIVITYBAR_PART;
                    break;
                default:
                    neighbour = Parts.EDITOR_PART;
            }
        }
        if (layoutService.isVisible(neighbour, activeWindow) || neighbour === Parts.EDITOR_PART) {
            return neighbour;
        }
        return this.findVisibleNeighbour(layoutService, neighbour, next);
    }
    focusNextOrPreviousPart(layoutService, editorService, next) {
        let currentlyFocusedPart;
        if (editorService.activeEditorPane?.hasFocus() || layoutService.hasFocus(Parts.EDITOR_PART)) {
            currentlyFocusedPart = Parts.EDITOR_PART;
        }
        else if (layoutService.hasFocus(Parts.ACTIVITYBAR_PART)) {
            currentlyFocusedPart = Parts.ACTIVITYBAR_PART;
        }
        else if (layoutService.hasFocus(Parts.STATUSBAR_PART)) {
            currentlyFocusedPart = Parts.STATUSBAR_PART;
        }
        else if (layoutService.hasFocus(Parts.SIDEBAR_PART)) {
            currentlyFocusedPart = Parts.SIDEBAR_PART;
        }
        else if (layoutService.hasFocus(Parts.AUXILIARYBAR_PART)) {
            currentlyFocusedPart = Parts.AUXILIARYBAR_PART;
        }
        else if (layoutService.hasFocus(Parts.PANEL_PART)) {
            currentlyFocusedPart = Parts.PANEL_PART;
        }
        layoutService.focusPart(currentlyFocusedPart ? this.findVisibleNeighbour(layoutService, currentlyFocusedPart, next) : Parts.EDITOR_PART, getActiveWindow());
    }
}
registerAction2(class extends BaseFocusAction {
    constructor() {
        super({
            id: 'workbench.action.focusNextPart',
            title: ( localize2(2754, 'Focus Next Part')),
            category: Categories.View,
            f1: true,
            keybinding: {
                primary: KeyCode.F6,
                weight: KeybindingWeight.WorkbenchContrib
            }
        }, true);
    }
});
registerAction2(class extends BaseFocusAction {
    constructor() {
        super({
            id: 'workbench.action.focusPreviousPart',
            title: ( localize2(2755, 'Focus Previous Part')),
            category: Categories.View,
            f1: true,
            keybinding: {
                primary: KeyMod.Shift | KeyCode.F6,
                weight: KeybindingWeight.WorkbenchContrib
            }
        }, false);
    }
});
