
import { localize } from '@codingame/monaco-vscode-api/vscode/vs/nls';
import { registerAction2 } from '@codingame/monaco-vscode-api/vscode/vs/platform/actions/common/actions';
import { Extensions } from '@codingame/monaco-vscode-api/vscode/vs/platform/configuration/common/configurationRegistry';
import { SyncDescriptor } from '@codingame/monaco-vscode-api/vscode/vs/platform/instantiation/common/descriptors';
import { Registry } from '@codingame/monaco-vscode-api/vscode/vs/platform/registry/common/platform';
import { EditorPaneDescriptor } from '@codingame/monaco-vscode-d0fb86d3-2a47-594e-955b-9a24631a7124-common/vscode/vs/workbench/browser/editor';
import { Extensions as Extensions$1, registerWorkbenchContribution2, WorkbenchPhase } from '@codingame/monaco-vscode-api/vscode/vs/workbench/common/contributions';
import { EditorExtensions } from '@codingame/monaco-vscode-api/vscode/vs/workbench/common/editor';
import { OpenResultResource, SetMixedLayout, SetColumnLayout, OpenMergeEditor, OpenBaseFile, ShowNonConflictingChanges, ShowHideBase, ShowHideTopBase, ShowHideCenterBase, GoToNextUnhandledConflict, GoToPreviousUnhandledConflict, ToggleActiveConflictInput1, ToggleActiveConflictInput2, CompareInput1WithBaseCommand, CompareInput2WithBaseCommand, AcceptAllInput1, AcceptAllInput2, ResetToBaseAndAutoMergeCommand, AcceptMerge, ResetCloseWithConflictsChoice, AcceptAllCombination, ToggleBetweenInputs } from './commands/commands.js';
import { MergeEditorCopyContentsToJSON, MergeEditorSaveContentsToFolder, MergeEditorLoadContentsFromFolder } from './commands/devCommands.js';
import { MergeEditorInput } from '@codingame/monaco-vscode-8ccb7637-50ea-5359-97bf-00015d7fe567-common/vscode/vs/workbench/contrib/mergeEditor/browser/mergeEditorInput';
import { MergeEditor, MergeEditorOpenHandlerContribution, MergeEditorResolverContribution } from './view/mergeEditor.js';
import { LifecyclePhase } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/lifecycle/common/lifecycle';
import { MergeEditorSerializer } from './mergeEditorSerializer.js';
import { AccessibleViewRegistry } from '@codingame/monaco-vscode-api/vscode/vs/platform/accessibility/browser/accessibleViewRegistry';
import { MergeEditorAccessibilityHelpProvider } from './mergeEditorAccessibilityHelp.js';

( Registry.as(EditorExtensions.EditorPane)).registerEditorPane(EditorPaneDescriptor.create(MergeEditor, MergeEditor.ID, ( localize(7800, "Merge Editor"))), [
    ( new SyncDescriptor(MergeEditorInput))
]);
( Registry.as(EditorExtensions.EditorFactory)).registerEditorSerializer(MergeEditorInput.ID, MergeEditorSerializer);
( Registry.as(Extensions.Configuration)).registerConfiguration({
    properties: {
        'mergeEditor.diffAlgorithm': {
            type: 'string',
            enum: ['legacy', 'advanced'],
            default: 'advanced',
            markdownEnumDescriptions: [
                ( localize(7801, "Uses the legacy diffing algorithm.")),
                ( localize(7802, "Uses the advanced diffing algorithm.")),
            ]
        },
        'mergeEditor.showDeletionMarkers': {
            type: 'boolean',
            default: true,
            description: 'Controls if deletions in base or one of the inputs should be indicated by a vertical bar.',
        },
    }
});
registerAction2(OpenResultResource);
registerAction2(SetMixedLayout);
registerAction2(SetColumnLayout);
registerAction2(OpenMergeEditor);
registerAction2(OpenBaseFile);
registerAction2(ShowNonConflictingChanges);
registerAction2(ShowHideBase);
registerAction2(ShowHideTopBase);
registerAction2(ShowHideCenterBase);
registerAction2(GoToNextUnhandledConflict);
registerAction2(GoToPreviousUnhandledConflict);
registerAction2(ToggleActiveConflictInput1);
registerAction2(ToggleActiveConflictInput2);
registerAction2(CompareInput1WithBaseCommand);
registerAction2(CompareInput2WithBaseCommand);
registerAction2(AcceptAllInput1);
registerAction2(AcceptAllInput2);
registerAction2(ResetToBaseAndAutoMergeCommand);
registerAction2(AcceptMerge);
registerAction2(ResetCloseWithConflictsChoice);
registerAction2(AcceptAllCombination);
registerAction2(ToggleBetweenInputs);
registerAction2(MergeEditorCopyContentsToJSON);
registerAction2(MergeEditorSaveContentsToFolder);
registerAction2(MergeEditorLoadContentsFromFolder);
( Registry
    .as(Extensions$1.Workbench))
    .registerWorkbenchContribution(MergeEditorOpenHandlerContribution, LifecyclePhase.Restored);
registerWorkbenchContribution2(MergeEditorResolverContribution.ID, MergeEditorResolverContribution, WorkbenchPhase.BlockStartup );
AccessibleViewRegistry.register(( new MergeEditorAccessibilityHelpProvider()));
